unit DXTex;

interface

uses
  Windows, SysUtils, Classes, Graphics, Direct3D, Direct3D9, Direct3D8,
  DirectDraw, DirectX, DXDraws, DXTexImg, DIB, Direct3DRM, DXTypes ;

type

  {  TDirect3DTexture2  }

  TDirect3DTexture2 = class
  private
    FDXDraw: TCustomDXDraw;
    FSrcImage: TObject;
    FImage: TDXTextureImage;
    FImage2: TDXTextureImage;
    FAutoFreeGraphic: Boolean;
    FSurface: TDirectDrawSurface;
    FTexture: IDirect3DTexture2;
    FTextureFormat: DirectDraw.TDDSurfaceDesc;
    FMipmap: Boolean;
    FTransparent: Boolean;
    FTransparentColor: TColorRef;
    FUseMipmap: Boolean;
    FUseColorKey: Boolean;
    FOnRestoreSurface: TNotifyEvent;
    FNeedLoadTexture: Boolean;
    FEnumTextureFormatFlag: Boolean;
    FD3DDevDesc: Direct3D.TD3DDeviceDesc;
    procedure DXDrawNotifyEvent(Sender: TCustomDXDraw; NotifyType: TDXDrawNotifyType);
    procedure SetDXDraw(ADXDraw: TCustomDXDraw);
    procedure LoadSubTexture(Dest: DirectDraw.IDirectDrawSurface4; SrcImage: TDXTextureImage);
    procedure SetColorKey;
    function GetMipmap: Boolean;
    function GetTexture: IDirect3DTexture2;
    function GetTransparent: Boolean;
    procedure SetMipmap(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparentColor(Value: TColorRef);
  protected
    procedure DoRestoreSurface; virtual;
  public
    constructor Create(ADXDraw: TCustomDXDraw; Graphic: TObject; AutoFreeGraphic: Boolean);
    constructor CreateFromFile(ADXDraw: TCustomDXDraw; const FileName: string);
    constructor CreateVideoTexture(ADXDraw: TCustomDXDraw; const TextureFormat: DirectDraw.TDDSurfaceDesc);
    destructor Destroy; override;
    procedure Finalize;
    procedure Load;
    procedure Initialize;
    property Mipmap: Boolean read GetMipmap write SetMipmap;
    property Surface: TDirectDrawSurface read FSurface;
    property Texture: IDirect3DTexture2 read GetTexture;
    property TextureFormat: DirectDraw.TDDSurfaceDesc read FTextureFormat;
    property Transparent: Boolean read GetTransparent write SetTransparent;
    property TransparentColor: TColorRef read FTransparentColor write SetTransparentColor;
    property OnRestoreSurface: TNotifyEvent read FOnRestoreSurface write FOnRestoreSurface;
  end;

implementation

function GetBitCount(i: Integer): Integer;
begin
  Result := 31;
  while (i>=0) and (((1 shl Result) and i)=0) do Dec(Result);
end;

function GetMaskBitCount(b: Integer): Integer;
var
  i: Integer;
begin
  i := 0;
  while (i<31) and (((1 shl i) and b)=0) do Inc(i);

  Result := 0;
  while ((1 shl i) and b)<>0 do
  begin
    Inc(i);
    Inc(Result);
  end;
end;

function Max(B1, B2: Integer): Integer;
begin
  if B1>=B2 then Result := B1 else Result := B2;
end;

function Min(B1, B2: Integer): Integer;
begin
  if B1<=B2 then Result := B1 else Result := B2;
end;

{  TDirect3DTexture2  }

constructor TDirect3DTexture2.Create(ADXDraw: TCustomDXDraw; Graphic: TObject;
  AutoFreeGraphic: Boolean);

  procedure SetDIB(DIB: TDIB);
  var
    i: Integer;
  begin
    FImage2 := TDXTextureImage.Create;
    if DIB.BitCount<=8 then
    begin
      FImage2.SetImage(DXTextureImageType_PaletteIndexedColor, DIB.Width, DIB.Height, DIB.BitCount,
        DIB.WidthBytes, DIB.NextLine, DIB.PBits, DIB.TopPBits, DIB.Size, False);

      FImage2.idx_index := dxtMakeChannel((1 shl DIB.BitCount)-1, True);
    end else
    begin
      FImage2.SetImage(DXTextureImageType_RGBColor, DIB.Width, DIB.Height, DIB.BitCount,
        DIB.WidthBytes, DIB.NextLine, DIB.PBits, DIB.TopPBits, DIB.Size, False);

      FImage2.rgb_red := dxtMakeChannel(DIB.NowPixelFormat.RBitMask, False);
      FImage2.rgb_green := dxtMakeChannel(DIB.NowPixelFormat.GBitMask, False);
      FImage2.rgb_blue := dxtMakeChannel(DIB.NowPixelFormat.BBitMask, False);

      i := DIB.NowPixelFormat.RBitCount+DIB.NowPixelFormat.GBitCount+DIB.NowPixelFormat.BBitCount;
      if i<DIB.BitCount then
        FImage2.rgb_alpha := dxtMakeChannel(((1 shl (DIB.BitCount-i))-1) shl i, False);
    end;

    FImage := FImage2;
  end;

begin
  inherited Create;
  FSrcImage := Graphic;
  FAutoFreeGraphic := AutoFreeGraphic;
  FNeedLoadTexture := True;

  if FSrcImage is TDXTextureImage then
    FImage := TDXTextureImage(FSrcImage)
  else if FSrcImage is TDIB then
    SetDIB(TDIB(FSrcImage))
  else if FSrcImage is TGraphic then
  begin
    FSrcImage := TDIB.Create;
    try
      TDIB(FSrcImage).Assign(TGraphic(Graphic));
      SetDIB(TDIB(FSrcImage));
    finally
      if FAutoFreeGraphic then
        Graphic.Free;
      FAutoFreeGraphic := True;
    end;
  end else
  if FSrcImage is TPicture then
  begin
    FSrcImage := TDIB.Create;
    try
      TDIB(FSrcImage).Assign(TPicture(Graphic).Graphic);
      SetDIB(TDIB(FSrcImage));                        
    finally
      if FAutoFreeGraphic then
        Graphic.Free;
      FAutoFreeGraphic := True;
    end;
  end else
    raise Exception.CreateFmt('OtBbN %s ͎gpł܂', [Graphic.ClassName]);

  FMipmap := FImage.SubGroupImageCount[DXTextureImageGroupType_Mipmap]>0;

  FTransparent := FImage.Transparent;
  case FImage.ImageType of
    DXTextureImageType_PaletteIndexedColor:
      begin
        FTransparentColor := PaletteIndex(dxtDecodeChannel(FImage.idx_index, FImage.TransparentColor));
      end;
    DXTextureImageType_RGBColor:
      begin
        FTransparentColor := RGB(dxtDecodeChannel(FImage.rgb_red, FImage.TransparentColor),
          dxtDecodeChannel(FImage.rgb_green, FImage.TransparentColor),
          dxtDecodeChannel(FImage.rgb_blue, FImage.TransparentColor));
      end;
  end;

  SetDXDraw(ADXDraw);
end;

constructor TDirect3DTexture2.CreateFromFile(ADXDraw: TCustomDXDraw; const FileName: string);
var
  FileName2: string;
  Image: TObject;
begin
  FileName2 := FileName;
  if ExtractFileDrive(FileName2)='' then
    FileName2 := ExtractFilePath(ParamStr(0))+FileName2;

  Image := nil;
  try
    {  TDXTextureImage  }
    Image := TDXTextureImage.Create;
    try
      TDXTextureImage(Image).LoadFromFile(FileName2);
    except
      Image.Free;
      Image := nil;
    end;

    {  TDIB  }
    if Image=nil then
    begin
      Image := TDIB.Create;
      try
        TDIB(Image).LoadFromFile(FileName2);
      except
        Image.Free;
        Image := nil;
      end;
    end;

    {  TPicture  }
    if Image=nil then
    begin
      Image := TPicture.Create;
      try
        TPicture(Image).LoadFromFile(FileName2);
      except
        Image.Free;
        Image := nil;
      end;
    end;

    if Image=nil then
      raise Exception.CreateFmt('t@C %s ǂݍނƂ͏o܂', [FileName]);
  except
    Image.Free;
    raise;
  end;

  Create(ADXDraw, Image, True);
end;

constructor TDirect3DTexture2.CreateVideoTexture(ADXDraw: TCustomDXDraw; const TextureFormat: DirectDraw.TDDSurfaceDesc);
begin
  inherited Create;
  FTextureFormat := TextureFormat;

  SetDXDraw(ADXDraw);
end;

destructor TDirect3DTexture2.Destroy;
begin
  SetDXDraw(nil);

  if FAutoFreeGraphic then
    FSrcImage.Free;
  FImage2.Free;
  inherited Destroy;
end;

procedure TDirect3DTexture2.DXDrawNotifyEvent(Sender: TCustomDXDraw;
  NotifyType: TDXDrawNotifyType);
begin
  case NotifyType of
    dxntDestroying:
        begin
          SetDXDraw(nil);
        end;
    dxntInitializeSurface:
        begin
          Initialize;
        end;        
    dxntFinalizeSurface:
        begin
          Finalize;
        end;
    dxntRestore:
        begin
          Load;
        end;
  end;
end;

procedure TDirect3DTexture2.SetDXDraw(ADXDraw: TCustomDXDraw);
begin
  if FDXDraw<>ADXDraw then
  begin
    if FDXDraw<>nil then
      FDXDraw.UnRegisterNotifyEvent(DXDrawNotifyEvent);

    FDXDraw := ADXDraw;

    if FDXDraw<>nil then
      FDXDraw.RegisterNotifyEvent(DXDrawNotifyEvent);
  end;
end;

procedure TDirect3DTexture2.DoRestoreSurface;
begin
  if Assigned(FOnRestoreSurface) then
    FOnRestoreSurface(Self);
end;

function TDirect3DTexture2.GetMipmap: Boolean;
begin
  if FSurface<>nil then
    Result := FUseMipmap
  else
    Result := FMipmap;
end;

function TDirect3DTexture2.GetTexture: IDirect3DTexture2;
begin
  Result := FTexture;
  if (Result<>nil) and FNeedLoadTexture then
    Load;
end;

function TDirect3DTexture2.GetTransparent: Boolean;
begin
  if FSurface<>nil then
    Result := FUseColorKey
  else
    Result := FTransparent;
end;

procedure TDirect3DTexture2.SetMipmap(Value: Boolean);
begin
  if FMipmap<>Value then
  begin
    FMipmap := Value;
    if FSurface<>nil then
      Initialize;
  end;
end;

procedure TDirect3DTexture2.SetTransparent(Value: Boolean);
begin
  if FTransparent<>Value then
  begin
    FTransparent := Value;
    if FSurface<>nil then
      SetColorKey;
  end;
end;

procedure TDirect3DTexture2.SetTransparentColor(Value: TColorRef);
begin
  if FTransparentColor<>Value then
  begin
    FTransparentColor := Value;
    if (FSurface<>nil) and FTransparent then
      SetColorKey;
  end;
end;

procedure TDirect3DTexture2.Finalize;
begin
  FTexture := nil;
  FSurface.Free; FSurface := nil;

  FUseColorKey := False;
  FUseMipmap := False;
  FNeedLoadTexture := False;
end;

const
  DDPF_PALETTEINDEXED = DDPF_PALETTEINDEXED1 or DDPF_PALETTEINDEXED2 or
    DDPF_PALETTEINDEXED4 or DDPF_PALETTEINDEXED8;

procedure TDirect3DTexture2.Initialize;

  function GetPaletteBitCount(ddsd: DirectDraw.TDDSurfaceDesc): Integer;
  begin
    if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED8<>0 then
      Result := 8
    else if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED4<>0 then
      Result := 4
    else if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED2<>0 then
      Result := 2
    else if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED1<>0 then
      Result := 1
    else
      Result := 0;
  end;

  function EnumTextureFormatCallback(const ddsd: DirectDraw.TDDSurfaceDesc;
    lParam: Pointer): HRESULT; stdcall;
  var
    tex: TDirect3DTexture2;

    procedure UseThisFormat;
    begin
      tex.FTextureFormat := ddsd;
      tex.FEnumTextureFormatFlag := True;
    end;

  var
    rgb_red, rgb_green, rgb_blue, rgb_alpha, idx_index: Integer;
    sum1, sum2: Integer;
  begin
    Result := DDENUMRET_OK;
    tex := lParam;

    {  eXg  }
    rgb_red := 0;
    rgb_green := 0;
    rgb_blue := 0;
    rgb_alpha := 0;
    idx_index := 0;

    case tex.FImage.ImageType of
      DXTextureImageType_RGBColor:
        begin
          {  RGB J[  }
          rgb_red := tex.FImage.rgb_red.bitcount;
          rgb_green := tex.FImage.rgb_green.bitcount;
          rgb_blue := tex.FImage.rgb_blue.bitcount;
          rgb_alpha := tex.FImage.rgb_alpha.bitcount;
          idx_index := 8;
        end;
      DXTextureImageType_PaletteIndexedColor:
        begin
          {  CfbNXJ[  }
          rgb_red := 8;
          rgb_green := 8;
          rgb_blue := 8;
          rgb_alpha := tex.FImage.idx_alpha.bitcount;
          idx_index := tex.FImage.idx_index.bitcount;
        end;
    end;

    {  ̌`͎g邩ׂ  }
    if ddsd.ddpfPixelFormat.dwFlags and DDPF_RGB=0 then Exit;

    case tex.FImage.ImageType of
      DXTextureImageType_RGBColor:
        begin
          if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED<>0 then Exit;
        end;
      DXTextureImageType_PaletteIndexedColor:
        begin
          if (ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED<>0) and
            (GetPaletteBitCount(ddsd)<idx_index) then Exit;
        end;
    end;

    {  g`IđI  }
    if tex.FEnumTextureFormatFlag then
    begin
      if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED<>0 then
      begin
        {  rbg`FbN  }
        if Abs(Integer(ddsd.ddpfPixelFormat.dwRGBBitCount)-idx_index)>
          Abs(Integer(tex.FTextureFormat.ddpfPixelFormat.dwRGBBitCount)-idx_index) then Exit;

        {  At@`l`FbN  }
        if rgb_alpha>0 then Exit;
      end else
      if ddsd.ddpfPixelFormat.dwFlags and DDPF_RGB<>0 then
      begin
        {  At@`lK{  }
        if (rgb_alpha>0) and (tex.FTextureFormat.ddpfPixelFormat.dwFlags and DDPF_ALPHAPIXELS=0) and
          (ddsd.ddpfPixelFormat.dwFlags and DDPF_ALPHAPIXELS<>0) then
        begin
          UseThisFormat;
          Exit;
        end;

        {  At@`l`FbN  }
        if (rgb_alpha>0) and (tex.FTextureFormat.ddpfPixelFormat.dwFlags and DDPF_ALPHAPIXELS<>0) and
          (ddsd.ddpfPixelFormat.dwFlags and DDPF_ALPHAPIXELS=0) then
        begin
          Exit;
        end;

        {  rbg`FbN  }
        if tex.FTextureFormat.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED=0 then
        begin
          sum1 := Sqr(GetMaskBitCount(ddsd.ddpfPixelFormat.dwRBitMask)-rgb_red)+
            Sqr(GetMaskBitCount(ddsd.ddpfPixelFormat.dwGBitMask)-rgb_green)+
            Sqr(GetMaskBitCount(ddsd.ddpfPixelFormat.dwBBitMask)-rgb_blue)+
            Sqr(GetMaskBitCount(ddsd.ddpfPixelFormat.dwRGBAlphaBitMask)-rgb_alpha);

          sum2 := Sqr(GetMaskBitCount(tex.FTextureFormat.ddpfPixelFormat.dwRBitMask)-rgb_red)+
            Sqr(GetMaskBitCount(tex.FTextureFormat.ddpfPixelFormat.dwGBitMask)-rgb_green)+
            Sqr(GetMaskBitCount(tex.FTextureFormat.ddpfPixelFormat.dwBBitMask)-rgb_blue)+
            Sqr(GetMaskBitCount(tex.FTextureFormat.ddpfPixelFormat.dwRGBAlphaBitMask)-rgb_alpha);

          if sum1>sum2 then Exit;
        end;
      end;
    end;

    UseThisFormat;
  end;

var
  Width, Height: Integer;
  ddsd: DirectDraw.TDDSurfaceDesc;
  PaletteCaps: DWORD;
  Palette: DirectDraw.IDirectDrawPalette;
  TempD3DDevDesc: Direct3D.TD3DDeviceDesc;
begin
  Finalize;
  try
    FDXDraw.D3DDevice.GetCaps(FD3DDevDesc, TempD3DDevDesc);

    if FImage<>nil then
    begin
      {  eXN`[̃TCY  }
      if FD3DDevDesc.dpcTriCaps.dwTextureCaps and D3DPTEXTURECAPS_POW2<>0 then
      begin
        {  eNX`̃TCY Sqr(n) ̂  }
        Width := Max(1 shl GetBitCount(FImage.Width), 1);
        Height := Max(1 shl GetBitCount(FImage.Height), 1);
      end else
      begin
        Width := FImage.Width;
        Height := FImage.Height;
      end;

      if FD3DDevDesc.dpcTriCaps.dwTextureCaps and D3DPTEXTURECAPS_SQUAREONLY<>0 then
      begin
        {  eNX`̃TCY͐`̂  }
        if Width<Height then Width := Height;
        Height := Width;
      end;

      if FD3DDevDesc.dwMinTextureWidth>0 then
        Width := Max(Width, FD3DDevDesc.dwMinTextureWidth);

      if FD3DDevDesc.dwMaxTextureWidth>0 then
        Width := Min(Width, FD3DDevDesc.dwMaxTextureWidth);

      if FD3DDevDesc.dwMinTextureHeight>0 then
        Height := Max(Height, FD3DDevDesc.dwMinTextureHeight);

      if FD3DDevDesc.dwMaxTextureHeight>0 then
        Height := Min(Height, FD3DDevDesc.dwMaxTextureHeight);
      
      FEnumTextureFormatFlag := False;
      FDXDraw.D3DDevice2.EnumTextureFormats(@EnumTextureFormatCallback, Self);

      FUseMipmap := FMipmap and (FTextureFormat.ddpfPixelFormat.dwRGBBitCount>8) and
        (FImage.SubGroupImageCount[DXTextureImageGroupType_Mipmap]>0) and (FDXDraw.DDraw.DriverCaps.ddsCaps.dwCaps and DDSCAPS_MIPMAP<>0);
    end else
    begin
      Width := FTextureFormat.dwWidth;
      Height := FTextureFormat.dwHeight;
    end;

    {  T[tF[X쐬  }
    FillChar(ddsd, SizeOf(ddsd), 0);
    with ddsd do
    begin
      dwSize := SizeOf(ddsd);
      dwFlags := DDSD_CAPS or DDSD_HEIGHT or DDSD_WIDTH or DDSD_PIXELFORMAT;
      ddsCaps.dwCaps := DDSCAPS_TEXTURE;
//      ddsCaps.dwCaps2 := DDSCAPS2_TEXTUREMANAGE;
      dwWidth := Width;
      dwHeight := Height;
      ddpfPixelFormat := FTextureFormat.ddpfPixelFormat;

      if FUseMipmap then
      begin
        dwFlags := dwFlags or DDSD_MIPMAPCOUNT;
        ddsCaps.dwCaps := ddsCaps.dwCaps or DDSCAPS_MIPMAP or DDSCAPS_COMPLEX;
        dwMipMapCount := FImage.SubGroupImageCount[DXTextureImageGroupType_Mipmap];
      end;
    end;

    FSurface := TDirectDrawSurface.Create(FDXDraw.DDraw);
    if not FSurface.CreateSurface(ddsd) then
      raise Exception.Create('eNX`쐬łȂ');

    {  pbg쐬  }
    if (FImage<>nil) and (FTextureFormat.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED<>0) then
    begin
      if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED8<>0 then
        PaletteCaps := DDPCAPS_8BIT or DDPCAPS_ALLOW256
      else if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED4<>0 then
        PaletteCaps := DDPCAPS_4BIT
      else if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED2<>0 then
        PaletteCaps := DDPCAPS_2BIT
      else if ddsd.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED1<>0 then
        PaletteCaps := DDPCAPS_1BIT
      else
        PaletteCaps := 0;

      if PaletteCaps<>0 then
      begin
        if FDXDraw.DDraw.IDraw.CreatePalette(PaletteCaps, @FImage.idx_palette, Palette, nil)<>0 then
          Exit;

        FSurface.ISurface.SetPalette(Palette);
      end;
    end;

    FNeedLoadTexture := True;
    
    {  IDirect3DTexture2 C^[tF[X쐬擾  }
    FTexture := FSurface.ISurface as IDirect3DTexture2;
  except
    Finalize;
    raise;
  end;
end;

procedure TDirect3DTexture2.Load;
const
  MipmapCaps: DirectDraw.TDDSCaps2 = (dwCaps: DDSCAPS_TEXTURE or DDSCAPS_MIPMAP);
var
  CurSurface, NextSurface: DirectDraw.IDirectDrawSurface4;
  Index: Integer;
  SrcImage: TDXTextureImage;
begin
  if FTexture=nil then
    Initialize;

  FNeedLoadTexture := False;
  FSurface.Restore;

  {  J[L[ݒ  }
  SetColorKey;

  {  T[tF[X։摜[h  }
  if FImage<>nil then
  begin
    CurSurface := FSurface.ISurface4;
    Index := 0;
    while CurSurface<>nil do
    begin
      SrcImage := FImage;
      if Index>0 then
      begin
        if Index-1>=FImage.SubGroupImageCount[DXTextureImageGroupType_Mipmap] then
          Break;
        SrcImage := FImage.SubGroupImages[DXTextureImageGroupType_Mipmap, Index-1];
      end;

      LoadSubTexture(CurSurface, SrcImage);

      if CurSurface.GetAttachedSurface(MipmapCaps, NextSurface)=0 then
        CurSurface := NextSurface
      else
        CurSurface := nil;

      Inc(Index);
    end;
  end else
    DoRestoreSurface;
end;

procedure TDirect3DTexture2.SetColorKey;
var
  ck: DirectDraw.PDDCOLORKEY;
begin
  FUseColorKey := False;

  if (FSurface<>nil) and FTransparent and (FD3DDevDesc.dpcTriCaps.dwTextureCaps and D3DPTEXTURECAPS_TRANSPARENCY<>0) then
  begin
    FillChar(ck, SizeOf(ck), 0);
    if FSurface.SurfaceDesc.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED<>0 then
    begin
      if FTransparentColor shr 24=$01 then
      begin
        {  pbgԍ  }
        ck.dwColorSpaceLowValue := FTransparentColor and $FF;
      end else
      if FImage<>nil then
      begin
        {  RGB l  }
        ck.dwColorSpaceLowValue := FImage.PaletteIndex(GetRValue(FTransparentColor), GetGValue(FTransparentColor), GetBValue(FTransparentColor));
      end else
        Exit;
    end else
    begin
      if (FImage<>nil) and (FImage.ImageType=DXTextureImageType_PaletteIndexedColor) and (FTransparentColor shr 24=$01) then
      begin
        {  pbgԍ  }
        ck.dwColorSpaceLowValue :=
          dxtEncodeChannel(dxtMakeChannel(FSurface.SurfaceDesc.ddpfPixelFormat.dwRBitMask, False), FImage.idx_palette[FTransparentColor and $FF].peRed) or
          dxtEncodeChannel(dxtMakeChannel(FSurface.SurfaceDesc.ddpfPixelFormat.dwGBitMask, False), FImage.idx_palette[FTransparentColor and $FF].peGreen) or
          dxtEncodeChannel(dxtMakeChannel(FSurface.SurfaceDesc.ddpfPixelFormat.dwBBitMask, False), FImage.idx_palette[FTransparentColor and $FF].peBlue);
      end else
      if FTransparentColor shr 24=$00 then
      begin
        {  RGB l  }
        ck.dwColorSpaceLowValue :=
          dxtEncodeChannel(dxtMakeChannel(FSurface.SurfaceDesc.ddpfPixelFormat.dwRBitMask, False), GetRValue(FTransparentColor)) or
          dxtEncodeChannel(dxtMakeChannel(FSurface.SurfaceDesc.ddpfPixelFormat.dwGBitMask, False), GetGValue(FTransparentColor)) or
          dxtEncodeChannel(dxtMakeChannel(FSurface.SurfaceDesc.ddpfPixelFormat.dwBBitMask, False), GetBValue(FTransparentColor));
      end else
        Exit;
    end;

    ck.dwColorSpaceHighValue := ck.dwColorSpaceLowValue;
    FSurface.ISurface.SetColorKey(DDCKEY_SRCBLT, ck);

    FUseColorKey := True;
  end;
end;

procedure TDirect3DTexture2.LoadSubTexture(Dest: DirectDraw.IDirectDrawSurface4; SrcImage: TDXTextureImage);
const
  Mask1: array[0..7] of DWORD = (1, 2, 4, 8, 16, 32, 64, 128);
  Mask2: array[0..3] of DWORD = (3, 12, 48, 192);
  Mask4: array[0..1] of DWORD = ($0F, $F0);
  Shift1: array[0..7] of DWORD = (0, 1, 2, 3, 4, 5, 6, 7);
  Shift2: array[0..3] of DWORD = (0, 2, 4, 6);
  Shift4: array[0..1] of DWORD = (0, 4);

  procedure SetPixel(const ddsd: DirectDraw.TDDSurfaceDesc2; x, y: Integer; c: DWORD);
  begin
    case ddsd.ddpfPixelFormat.dwRGBBitCount of
      1 : PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x div 8)^ :=
            (PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x div 8)^ and (not Mask1[x mod 8])) or (c shl Shift1[x mod 8]);
      2 : PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x div 4)^ :=
            (PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x div 4)^ and (not Mask2[x mod 4])) or (c shl Shift2[x mod 4]);
      4 : PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x div 2)^ :=
            (PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x div 2)^ and (not Mask4[x mod 2])) or (c shl Shift4[x mod 2]);
      8 : PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x)^ := c;
      16: PWord(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x*2)^ := c;
      24: begin
            PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x*3)^ := c shr 0;
            PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x*3+1)^ := c shr 8;
            PByte(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x*3+2)^ := c shr 16;
          end;   
      32: PDWORD(Integer(ddsd.lpSurface)+ddsd.lPitch*y+x*4)^ := c;
    end;
  end;

  procedure LoadTexture_IndexToIndex;
  var
    ddsd: DirectDraw.TDDSurfaceDesc2;
    x, y: Integer;
  begin
    ddsd.dwSize := SizeOf(ddsd);
    if Dest.Lock(nil, ddsd, DDLOCK_WAIT, 0)=0 then
    begin
      try
        for y:=0 to ddsd.dwHeight-1 do
          for x:=0 to ddsd.dwWidth-1 do
            SetPixel(ddsd, x, y, dxtDecodeChannel(SrcImage.idx_index, SrcImage.Pixels[x, y]));
      finally
        Dest.UnLock(ddsd.lpSurface);
      end;
    end;
  end;

  procedure LoadTexture_IndexToRGB;
  var
    ddsd: DirectDraw.TDDSurfaceDesc2;
    x, y: Integer;
    c, cIdx, cA: DWORD;
    dest_red_fmt, dest_green_fmt, dest_blue_fmt, dest_alpha_fmt: TDXTextureImageChannel;
  begin
    ddsd.dwSize := SizeOf(ddsd);
    if Dest.Lock(nil, ddsd, DDLOCK_WAIT, 0)=0 then
    begin
      try
        dest_red_fmt := dxtMakeChannel(ddsd.ddpfPixelFormat.dwRBitMask, False);
        dest_green_fmt := dxtMakeChannel(ddsd.ddpfPixelFormat.dwGBitMask, False);
        dest_blue_fmt := dxtMakeChannel(ddsd.ddpfPixelFormat.dwBBitMask, False);
        dest_alpha_fmt := dxtMakeChannel(ddsd.ddpfPixelFormat.dwRGBAlphaBitMask, False);

        if SrcImage.idx_alpha.mask<>0 then
        begin
          for y:=0 to ddsd.dwHeight-1 do
            for x:=0 to ddsd.dwWidth-1 do
            begin
              c := SrcImage.Pixels[x, y];
              cIdx := dxtDecodeChannel(SrcImage.idx_index, c);

              c := dxtEncodeChannel(dest_red_fmt, SrcImage.idx_palette[cIdx].peRed) or
                dxtEncodeChannel(dest_green_fmt, SrcImage.idx_palette[cIdx].peGreen) or
                dxtEncodeChannel(dest_blue_fmt, SrcImage.idx_palette[cIdx].peBlue) or
                dxtEncodeChannel(dest_alpha_fmt, dxtDecodeChannel(SrcImage.idx_alpha, c));

              SetPixel(ddsd, x, y, c);
            end;
        end else
        begin
          cA := dxtEncodeChannel(dest_alpha_fmt, 255);

          for y:=0 to ddsd.dwHeight-1 do
            for x:=0 to ddsd.dwWidth-1 do
            begin
              c := SrcImage.Pixels[x, y];
              cIdx := dxtDecodeChannel(SrcImage.idx_index, c);

              c := dxtEncodeChannel(dest_red_fmt, SrcImage.idx_palette[cIdx].peRed) or
                dxtEncodeChannel(dest_green_fmt, SrcImage.idx_palette[cIdx].peGreen) or
                dxtEncodeChannel(dest_blue_fmt, SrcImage.idx_palette[cIdx].peBlue) or cA;

              SetPixel(ddsd, x, y, c);
            end;
        end;
      finally
        Dest.UnLock(ddsd.lpSurface);
      end;
    end;
  end;

  procedure LoadTexture_RGBToRGB;
  var
    ddsd: DirectDraw.TDDSurfaceDesc2;
    x, y: Integer;
    c, cA: DWORD;
    dest_red_fmt, dest_green_fmt, dest_blue_fmt, dest_alpha_fmt: TDXTextureImageChannel;
  begin
    ddsd.dwSize := SizeOf(ddsd);
    if Dest.Lock(nil, ddsd, DDLOCK_WAIT, 0)=0 then
    begin
      try
        dest_red_fmt := dxtMakeChannel(ddsd.ddpfPixelFormat.dwRBitMask, False);
        dest_green_fmt := dxtMakeChannel(ddsd.ddpfPixelFormat.dwGBitMask, False);
        dest_blue_fmt := dxtMakeChannel(ddsd.ddpfPixelFormat.dwBBitMask, False);
        dest_alpha_fmt := dxtMakeChannel(ddsd.ddpfPixelFormat.dwRGBAlphaBitMask, False);

        if SrcImage.rgb_alpha.mask<>0 then
        begin
          for y:=0 to ddsd.dwHeight-1 do
            for x:=0 to ddsd.dwWidth-1 do
            begin
              c := SrcImage.Pixels[x, y];

              c := dxtEncodeChannel(dest_red_fmt, dxtDecodeChannel(SrcImage.rgb_red, c)) or
                dxtEncodeChannel(dest_green_fmt, dxtDecodeChannel(SrcImage.rgb_green, c)) or
                dxtEncodeChannel(dest_blue_fmt, dxtDecodeChannel(SrcImage.rgb_blue, c)) or
                dxtEncodeChannel(dest_alpha_fmt, dxtDecodeChannel(SrcImage.rgb_alpha, c));

              SetPixel(ddsd, x, y, c);
            end;
        end else
        begin
          cA := dxtEncodeChannel(dest_alpha_fmt, 255);

          for y:=0 to ddsd.dwHeight-1 do
            for x:=0 to ddsd.dwWidth-1 do
            begin                      
              c := SrcImage.Pixels[x, y];

              c := dxtEncodeChannel(dest_red_fmt, dxtDecodeChannel(SrcImage.rgb_red, c)) or
                dxtEncodeChannel(dest_green_fmt, dxtDecodeChannel(SrcImage.rgb_green, c)) or
                dxtEncodeChannel(dest_blue_fmt, dxtDecodeChannel(SrcImage.rgb_blue, c)) or cA;

              SetPixel(ddsd, x, y, c);
            end;
        end;
      finally
        Dest.UnLock(ddsd.lpSurface);
      end;
    end;
  end;

var
  SurfaceDesc: DirectDraw.TDDSurfaceDesc2;
begin
  SurfaceDesc.dwSize := SizeOf(SurfaceDesc);
  Dest.GetSurfaceDesc(SurfaceDesc);

  if SurfaceDesc.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED<>0 then
  begin
    case SrcImage.ImageType of
      DXTextureImageType_PaletteIndexedColor: LoadTexture_IndexToIndex;
      DXTextureImageType_RGBColor           : ;
    end;
  end else if SurfaceDesc.ddpfPixelFormat.dwFlags and DDPF_RGB<>0 then
  begin
    case SrcImage.ImageType of
      DXTextureImageType_PaletteIndexedColor: LoadTexture_IndexToRGB;
      DXTextureImageType_RGBColor           : LoadTexture_RGBToRGB;
    end;
  end;
end;

end.
