(*----------------------------------------------------------------------------*
 *  DirectX 9 SDK December 2004 C++ common framework adaptation for Delphi    *
 *  by Alexey Barkovoy (e-mail: clootie@ixbt.com)                             *
 *                                                                            *
 *  Desc: Direct3D part of framework.                                         *
 *  Supported compilers: Delphi 5,6,7,9 FreePascal 2.0                        *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru                                                *
 *     http://sourceforge.net/projects/delphi-dx9sdk                          * 
 *----------------------------------------------------------------------------*
 *  $Id: DXUTEffectMap.pas,v 1.4 2005/01/18 15:00:29 clootie Exp $
 *----------------------------------------------------------------------------*)
{******************************************************************************}
{                                                                              }
{ Obtained through: Joint Endeavour of Delphi Innovators (Project JEDI)        }
{                                                                              }
{ The contents of this file are used with permission, subject to the Mozilla   }
{ Public License Version 1.1 (the "License"); you may not use this file except }
{ in compliance with the License. You may obtain a copy of the License at      }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                      }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ Alternatively, the contents of this file may be used under the terms of the  }
{ GNU Lesser General Public License (the  "LGPL License"), in which case the   }
{ provisions of the LGPL License are applicable instead of those above.        }
{ If you wish to allow use of your version of this file only under the terms   }
{ of the LGPL License and not to allow others to use your version of this file }
{ under the MPL, indicate your decision by deleting  the provisions above and  }
{ replace  them with the notice and other provisions required by the LGPL      }
{ License.  If you do not delete the provisions above, a recipient may use     }
{ your version of this file under either the MPL or the LGPL License.          }
{                                                                              }
{ For more information about the LGPL: http://www.gnu.org/copyleft/lesser.html }
{                                                                              }
{******************************************************************************}

//-----------------------------------------------------------------------------
// File: DXUTEffectMap.h
//
// Desc: Maps the set of standard semantics and annotations to a collection
//       of ID3DXEffect objects.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit DXUTEffectMap;

interface

uses
  Windows, Direct3D9, D3DX9;

const
  MAX_INDEX {int} = 5;

type
  TDXUTSemantic =
  (
    DXUT_UNKNOWN_SEMANTIC,

    DXUT_Diffuse,
    DXUT_Specular,
    DXUT_Ambient,
    DXUT_SpecularPower,
    DXUT_Emissive,
    DXUT_Normal,
    DXUT_Height,
    DXUT_Refraction,
    DXUT_Opacity,
    DXUT_Environment,
    DXUT_EnvironmentNormal,
    DXUT_Fresnel,

    DXUT_World,
    DXUT_WorldInverse,
    DXUT_WorldInverseTranspose,
    DXUT_WorldView,
    DXUT_WorldViewInverse,
    DXUT_WorldViewInverseTranspose,
    DXUT_WorldViewProjection,
    DXUT_WorldViewProjectionInverse,
    DXUT_WorldViewProjectionInverseTranspose,
    DXUT_View,
    DXUT_ViewInverse,
    DXUT_ViewInverseTranspose,
    DXUT_ViewProjection,
    DXUT_ViewProjectionInverse,
    DXUT_ViewProjectionInverseTranspose,
    DXUT_Projection,
    DXUT_ProjectionInverse,
    DXUT_ProjectionInverseTranspose,

    DXUT_RenderTargetDimensions,
    DXUT_RenderTargetClipping,
    DXUT_Time,
    DXUT_LastTime,
    DXUT_ElapsedTime,
    DXUT_Position,
    DXUT_Direction,
    DXUT_BoundingCenter,
    DXUT_BoundingSphereSize,
    DXUT_BoundingSphereMin,
    DXUT_BoundingSphereMax,
    DXUT_BoundingBoxSize,
    DXUT_BoundingBoxMin,
    DXUT_BoundingBoxMax,
    DXUT_Attenuation,
    DXUT_RenderColorTarget,
    DXUT_RenderDepthStencilTarget,
    DXUT_UnitsScale,
    DXUT_StandardsGlobal
    // NUM_DXUT_SEMANTICS
  );

const
  NUM_DXUT_SEMANTICS = Integer(High(TDXUTSemantic))+1;

type  
  TDXUTObject =
  (
    DXUT_UNKNOWN_OBJECT,

    DXUT_Geometry,
    DXUT_Light,
    DXUT_Camera,
    DXUT_Frame

    // NUM_DXUT_OBJECTS
  );

const
  NUM_DXUT_OBJECTS = Integer(High(TDXUTObject))+1;

type
  PParamList = ^TParamList;
  TParamList = record
    pEffect: ID3DXEffect;
    ahParameters: array of TD3DXHandle;
  end;

  TParamListArray = array of TParamList;

procedure TParamList_Reset(pl: TParamList);


type
  CDXUTEffectMap = class
  public
    constructor Create; { Reset(); }
    destructor Destroy; override;{ Reset(); }

    procedure Reset;

    function AddEffect(pEffect: ID3DXEffect): HRESULT;
    function RemoveEffect(pEffect: ID3DXEffect): HRESULT;

    function SetStandardParameter(const strSemantic, strObject: PWideChar;
        dwObjectIndex: DWORD; pData: PSingle; dwDataLen: DWORD;
        const strType: PWideChar = nil; const strUnits: PWideChar = nil;
        const strSpace: PWideChar = nil): HRESULT; overload;
    function SetStandardParameter(const strSemantic, strObject: PWideChar;
        dwObjectIndex: DWORD; pData: PSingle; dwDataLen: DWORD;
        const strType: WideString = ''; const strUnits: WideString = '';
        const strSpace: WideString = ''): HRESULT; overload;
    function SetStandardParameter(eSemantic: TDXUTSemantic; eObject: TDXUTObject;
        dwObjectIndex: DWORD; pData: PSingle; dwDataLen: DWORD;
        const strType: PWideChar = nil; const strUnits: PWideChar = nil;
        const strSpace: PWideChar = nil): HRESULT; overload;
    function SetStandardParameter(eSemantic: TDXUTSemantic; eObject: TDXUTObject;
        dwObjectIndex: DWORD; pData: PSingle; dwDataLen: DWORD;
        const strType: WideString = ''; const strUnits: WideString = '';
        const strSpace: WideString = ''): HRESULT; overload;

    function SetWorldMatrix(const pWorldMatrix: TD3DXMatrixA16; const strUnits: PWideChar = nil): HRESULT;
    function SetViewMatrix(const pViewMatrix: TD3DXMatrixA16; const strUnits: PWideChar = nil): HRESULT;
    function SetProjectionMatrix(const pProjectionMatrix: TD3DXMatrixA16): HRESULT;

    {static} function StringToSemantic(const cstrSemantic: PAnsiChar): TDXUTSemantic; overload;
    {static} function StringToSemantic(const strSemantic: PWideChar): TDXUTSemantic; overload;
    {static} function StringToObject(const cstrObject: PAnsiChar): TDXUTObject; overload;
    {static} function StringToObject(const strObject: PWideChar): TDXUTObject; overload;

  private
    m_matWorld: TD3DXMatrixA16;
    m_matView: TD3DXMatrixA16;
    m_matProjection: TD3DXMatrixA16;

    // Database of DXUTEffect object parameter handles which are indexed accoring to
    // Semantic, Object annotation, index, and containing mesh pointer
    m_Bindings: array[Low(TDXUTSemantic)..High(TDXUTSemantic), Low(TDXUTObject)..High(TDXUTObject), 0..MAX_INDEX-1] of TParamListArray;

    function UpdateTransforms(eSemantic: TDXUTSemantic; const strUnits: WideString = 'm'): HRESULT; 
  end;

implementation

uses
  SysUtils, DXUTcore;

//-----------------------------------------------------------------------------
// File: DXUTEffectMap.cpp
//
// Desc: Maps the set of standard semantics and annotations to a collection
//       of ID3DXEffect objects.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------


(*//-------------------------------------------------------------------------------------
inline bool StringBegins( const char* strTest, const char* strStartsWith )
{
    return ( 0 == _strnicmp( strTest, strStartsWith, strlen(strStartsWith) ) );
}


//-------------------------------------------------------------------------------------
inline bool StringEquals( const char* strTest, const char* strEquals )
{
    return( 0 == _stricmp( strTest, strEquals ) );
}*)


//-------------------------------------------------------------------------------------
procedure RemoveTrailingNumber(strWithoutNumber: PAnsiChar; dwBufLen: DWORD; const str: PChar);{$IFDEF SUPPORTS_INLINE} inline;{$ENDIF}
var
  strIndex: PChar;
begin
  StringCchCopy(strWithoutNumber, dwBufLen, str);

  strIndex := strWithoutNumber + StrLen(strWithoutNumber);

  // If there is a digit at the end of the semantic discard it
  while (strIndex-1)^ in ['0'..'9'] do Dec(strIndex);

  strIndex^ := #0;
end;





//-------------------------------------------------------------------------------------
constructor CDXUTEffectMap.Create;
begin
  Reset;
end;


//-------------------------------------------------------------------------------------
destructor CDXUTEffectMap.Destroy;
begin
  Reset;
  inherited;
end;


//-------------------------------------------------------------------------------------
{static} function CDXUTEffectMap.StringToSemantic(const cstrSemantic: PAnsiChar): TDXUTSemantic;
var
  strSemantic: array[0..MAX_PATH+2] of AnsiChar;
begin
  strSemantic:= '';

  RemoveTrailingNumber(strSemantic, MAX_PATH, cstrSemantic);

  Result:= DXUT_UNKNOWN_SEMANTIC;

  if strSemantic = 'Diffuse'                               then Result:= DXUT_Diffuse;
  if strSemantic = 'Specular'                              then Result:= DXUT_Specular;
  if strSemantic = 'Ambient'                               then Result:= DXUT_Ambient;
  if strSemantic = 'SpecularPower'                         then Result:= DXUT_SpecularPower;
  if strSemantic = 'Emissive'                              then Result:= DXUT_Emissive;
  if strSemantic = 'Normal'                                then Result:= DXUT_Normal;
  if strSemantic = 'Height'                                then Result:= DXUT_Height;
  if strSemantic = 'Refraction'                            then Result:= DXUT_Refraction;
  if strSemantic = 'Opacity'                               then Result:= DXUT_Opacity;
  if strSemantic = 'Environment'                           then Result:= DXUT_Environment;
  if strSemantic = 'EnvironmentNormal'                     then Result:= DXUT_EnvironmentNormal;
  if strSemantic = 'Fresnel'                               then Result:= DXUT_Fresnel;

  if strSemantic = 'World'                                 then Result:= DXUT_World;
  if strSemantic = 'WorldInverse'                          then Result:= DXUT_WorldInverse;
  if strSemantic = 'WorldInverseTranspose'                 then Result:= DXUT_WorldInverseTranspose;
  if strSemantic = 'WorldView'                             then Result:= DXUT_WorldView;
  if strSemantic = 'WorldViewInverse'                      then Result:= DXUT_WorldViewInverse;
  if strSemantic = 'WorldViewInverseTranspose'             then Result:= DXUT_WorldViewInverseTranspose;
  if strSemantic = 'WorldViewProjection'                   then Result:= DXUT_WorldViewProjection;
  if strSemantic = 'WorldViewProjectionInverse'            then Result:= DXUT_WorldViewProjectionInverse;
  if strSemantic = 'WorldViewProjectionInverseTranspose'   then Result:= DXUT_WorldViewProjectionInverseTranspose;
  if strSemantic = 'View'                                  then Result:= DXUT_View;
  if strSemantic = 'ViewInverse'                           then Result:= DXUT_ViewInverse;
  if strSemantic = 'ViewInverseTranspose'                  then Result:= DXUT_ViewInverseTranspose;
  if strSemantic = 'ViewProjection'                        then Result:= DXUT_ViewProjection;
  if strSemantic = 'ViewProjectionInverse'                 then Result:= DXUT_ViewProjectionInverse;
  if strSemantic = 'ViewProjectionInverseTranspose'        then Result:= DXUT_ViewProjectionInverseTranspose;
  if strSemantic = 'Projection'                            then Result:= DXUT_Projection;
  if strSemantic = 'ProjectionInverse'                     then Result:= DXUT_ProjectionInverse;
  if strSemantic = 'ProjectionInverseTranspose'            then Result:= DXUT_ProjectionInverseTranspose;

  if strSemantic = 'RenderTargetDimensions'                then Result:= DXUT_RenderTargetDimensions;
  if strSemantic = 'RenderTargetClipping'                  then Result:= DXUT_RenderTargetClipping;
  if strSemantic = 'Time'                                  then Result:= DXUT_Time;
  if strSemantic = 'LastTime'                              then Result:= DXUT_LastTime;
  if strSemantic = 'ElapsedTime'                           then Result:= DXUT_ElapsedTime;
  if strSemantic = 'Position'                              then Result:= DXUT_Position;
  if strSemantic = 'Direction'                             then Result:= DXUT_Direction;
  if strSemantic = 'BoundingCenter'                        then Result:= DXUT_BoundingCenter;
  if strSemantic = 'BoundingSphereSize'                    then Result:= DXUT_BoundingSphereSize;
  if strSemantic = 'BoundingSphereMin'                     then Result:= DXUT_BoundingSphereMin;
  if strSemantic = 'BoundingSphereMax'                     then Result:= DXUT_BoundingSphereMax;
  if strSemantic = 'BoundingBoxSize'                       then Result:= DXUT_BoundingBoxSize;
  if strSemantic = 'BoundingBoxMin'                        then Result:= DXUT_BoundingBoxMin;
  if strSemantic = 'BoundingBoxMax'                        then Result:= DXUT_BoundingBoxMax;
  if strSemantic = 'Attenuation'                           then Result:= DXUT_Attenuation;
  if strSemantic = 'RenderColorTarget'                     then Result:= DXUT_RenderColorTarget;
  if strSemantic = 'RenderDepthStencilTarget'              then Result:= DXUT_RenderDepthStencilTarget;
  if strSemantic = 'UnitsScale'                            then Result:= DXUT_UnitsScale;
  if strSemantic = 'StandardsGlobal'                       then Result:= DXUT_StandardsGlobal;
end;


//-------------------------------------------------------------------------------------
{static} function CDXUTEffectMap.StringToSemantic(const strSemantic: PWideChar): TDXUTSemantic;
var
  cstr: array[0..MAX_PATH+2] of Char;
begin
  cstr := '';
  WideCharToMultiByte(CP_ACP, 0, strSemantic, -1, cstr, MAX_PATH, nil, nil);
  Result:= StringToSemantic(cstr);
end;


//-------------------------------------------------------------------------------------
{static} function CDXUTEffectMap.StringToObject(const cstrObject: PAnsiChar): TDXUTObject;
var
  strObject: array[0..MAX_PATH+2] of Char;
begin
  strObject := '';
  RemoveTrailingNumber(strObject, MAX_PATH, cstrObject);

  Result:= DXUT_UNKNOWN_OBJECT;

  if strObject = 'Geometry' then Result:= DXUT_Geometry;
  if strObject = 'Light'    then Result:= DXUT_Light;
  if strObject = 'Camera'   then Result:= DXUT_Camera;
  if strObject = 'Frame'    then Result:= DXUT_Frame;
end;


//-------------------------------------------------------------------------------------
{static} function CDXUTEffectMap.StringToObject(const strObject: PWideChar): TDXUTObject; 
var
  cstr: array[0..MAX_PATH+2] of Char;
begin
  cstr := '';
  WideCharToMultiByte(CP_ACP, 0, strObject, -1, cstr, MAX_PATH, nil, nil);
  Result:= StringToObject(cstr);
end;


//-------------------------------------------------------------------------------------
procedure CDXUTEffectMap.Reset;
var
  iSemantic: TDXUTSemantic;
  iObject: TDXUTObject;
  iIndex, iParamList: Integer;
  pBinding: TParamListArray;
begin
	D3DXMatrixIdentity(m_matWorld);
  D3DXMatrixIdentity(m_matView);
  D3DXMatrixIdentity(m_matProjection);

	// Reset all the stored parameter lists
	for iSemantic := Low(TDXUTSemantic) to High(TDXUTSemantic) do
	begin
		for iObject := Low(TDXUTObject) to High(TDXUTObject) do
		begin
			for iIndex := 0 to MAX_INDEX-1 do 
			begin
        pBinding := @m_Bindings[iSemantic][iObject][iIndex];

				// Clear nested arrays first
				for iParamList := 0 to Length(pBinding) - 1 do TParamList_Reset(pBinding[iParamList]);

				// Remove all the bound parameter lists
				pBinding:= nil; // RemoveAll;
			end;
		end;
	end;
end;


//-------------------------------------------------------------------------------------
function CDXUTEffectMap.SetStandardParameter(const strSemantic, strObject: PWideChar;
        dwObjectIndex: DWORD; pData: PSingle; dwDataLen: DWORD;
        const strType: PWideChar = nil; const strUnits: PWideChar = nil;
        const strSpace: PWideChar = nil): HRESULT; 
var
  eSemantic: TDXUTSemantic;
  eObject: TDXUTObject;
begin
  Result:= E_FAIL;

  // Map the semantic to the standard set
  eSemantic := StringToSemantic(strSemantic);
  if (eSemantic = DXUT_UNKNOWN_SEMANTIC) then Exit;

  // Map the object to the standard set
  eObject := StringToObject(strObject);
  if (eObject = DXUT_UNKNOWN_OBJECT) then Exit;

  Result:= SetStandardParameter(eSemantic, eObject, dwObjectIndex, pData,
               dwDataLen, strType, strUnits, strSpace);
end;


function CDXUTEffectMap.SetStandardParameter(const strSemantic,
  strObject: PWideChar; dwObjectIndex: DWORD; pData: PSingle;
  dwDataLen: DWORD; const strType, strUnits,
  strSpace: WideString): HRESULT;
begin
  Result:= SetStandardParameter(strSemantic, strObject, dwObjectIndex, pData, dwDataLen,
                                PWideChar(strType), PWideChar(strUnits), PWideChar(strSpace));
end;

//-------------------------------------------------------------------------------------
function CDXUTEffectMap.SetStandardParameter(eSemantic: TDXUTSemantic; eObject: TDXUTObject;
        dwObjectIndex: DWORD; pData: PSingle; dwDataLen: DWORD;
        const strType: PWideChar = nil; const strUnits: PWideChar = nil;
        const strSpace: PWideChar = nil): HRESULT;
var
  pBindings: TParamListArray;
  iList: Integer;
  paramList: PParamList;
  iParam: Integer;
begin
  pBindings:= nil;
  
  // TODO: remove index limits
  if (dwObjectIndex >= MAX_INDEX) then
  begin
    Result:= E_INVALIDARG;
    Exit;
  end;

  // TODO: handle unit and space conversions

  // Retrieve the interested handles
  pBindings := m_Bindings[eSemantic][eObject][dwObjectIndex];

  for iList := 0 to Length(pBindings) - 1 do
  begin
    paramList := @pBindings[iList];

    for iParam := 0 to Length(paramList.ahParameters) - 1 do
    begin
      Result:= paramList.pEffect.SetFloatArray(paramList.ahParameters[iParam], pData, dwDataLen);
      if V_Failed(Result) then Exit;
    end;
  end;

  Result:= S_OK;
end;

function CDXUTEffectMap.SetStandardParameter(eSemantic: TDXUTSemantic;
  eObject: TDXUTObject; dwObjectIndex: DWORD; pData: PSingle;
  dwDataLen: DWORD; const strType, strUnits,
  strSpace: WideString): HRESULT;
begin
  Result := SetStandardParameter(eSemantic, eObject, dwObjectIndex, pData, dwDataLen,
                                 PWideChar(strType), PWideChar(strUnits), PWideChar(strSpace));
end;

procedure TParamList_Reset(pl: TParamList);
begin
  with pl do
  begin
    SAFE_RELEASE(pEffect);
	  ahParameters:= nil; // .RemoveAll;
  end;
end;


//-------------------------------------------------------------------------------------
// Investigates all the parameters, looking at semantics and annotations and placing 
// handles to these parameters within the internal database.
//-------------------------------------------------------------------------------------
function CDXUTEffectMap.AddEffect(pEffect: ID3DXEffect): HRESULT;
const
  cstrObjectC: PChar = 'Geometry';
var
  descEffect: TD3DXEffectDesc;
  iParam: Integer;
  hParameter: TD3DXHandle;
  desc: TD3DXParameterDesc;
  eSemantic: TDXUTSemantic;
  hAnnotation: TD3DXHandle;
  cstrObject: PChar;
  eObject: TDXUTObject;
  index: Integer;
  strIndex: PChar;
  pBindings: TParamListArray;
  bBound: Boolean;
  i: Integer;
  l: Integer;
  newParamList: TParamList;
begin
	if (pEffect = nil) then
  begin
		Result:= E_INVALIDARG;
    Exit;
  end;

  // Get the number of parameters
  Result:= pEffect.GetDesc(descEffect);
  if V_Failed(Result) then Exit;

  // Enumerate the parameters
  for iParam := 0 to descEffect.Parameters - 1 do
  begin
    // Retrieve param
    hParameter := pEffect.GetParameter(nil, iParam);
    if (hParameter = nil) then
    begin
      Result:= E_FAIL;
      Exit;
    end;

    // Grab description
    Result := pEffect.GetParameterDesc(hParameter, desc);
    if V_Failed(Result) then Exit;

    // If this parameter doesn't have a semantic, skip to the next parameter
    if (desc.Semantic = nil) then Continue;

    // Map the semantic to the standard set
    eSemantic := StringToSemantic(desc.Semantic);
    if (eSemantic = DXUT_UNKNOWN_SEMANTIC) then Continue;

    // Get the object annotation
    cstrObject:= cstrObjectC;
    hAnnotation := pEffect.GetAnnotationByName(hParameter, 'Object');
    if (hAnnotation <> nil) then
    begin
      Result := pEffect.GetString(hAnnotation, cstrObject);
      if V_Failed(Result) then Exit;
    end;

    // Map the object to the standard set
    eObject := StringToObject(cstrObject);
    if (eObject = DXUT_UNKNOWN_OBJECT) then Continue;

    // Extract the index from the semantic
    index := 0;
    strIndex := desc.Semantic + StrLen(desc.Semantic)-1;

    // If there is a digit at the end of the semantic, locate the beginning of the index
    // and convert to an integer
    if strIndex^ in ['0'..'9'] then
    begin
      while (strIndex-1)^ in ['0'..'9'] do Dec(strIndex);
      index := StrToInt(strIndex);
    end;

    // Check whether index is out of bounds
    if (index < 0) or (index >= MAX_INDEX) then Continue;

    // Store the handle
    pBindings := @m_Bindings[eSemantic][eObject][index];

    bBound := False;
    for i := 0 to Length(pBindings) - 1 do 
    begin
      if (pBindings[i].pEffect = pEffect) then
      begin
        // Found the containing effect for this parameter in the list, add the new handle
        // pBindings[i].ahParameters.Add(hParameter);
        l:= Length(pBindings[i].ahParameters);
        SetLength(pBindings[i].ahParameters, l+1);
        pBindings[i].ahParameters[l]:= hParameter;
        bBound := True;
        Break;
      end;
    end;

    if not bBound then
    begin
      // This parameter belongs to a new effect
      newParamList.pEffect := pEffect;
      // newParamList.ahParameters.Add(hParameter);
      l:= Length(newParamList.ahParameters);
      SetLength(newParamList.ahParameters, l+1);
      newParamList.ahParameters[l]:= hParameter;
      // pBindings.Add(newParamList);
      l:= Length(pBindings);
      SetLength(pBindings, l+1);
      pBindings[l]:= newParamList;
    end;
  end;

  Result:= S_OK;
end;


//-------------------------------------------------------------------------------------
// Removes all instances of this effect from the binding list
//-------------------------------------------------------------------------------------
function CDXUTEffectMap.RemoveEffect(pEffect: ID3DXEffect): HRESULT;
var
  iSemantic: TDXUTSemantic;
  iObject: TDXUTObject;
  iIndex: Integer;
  pBinding: TParamListArray;
  iParamList: Integer;
  rParamList: PParamList;
begin
  pBinding := nil;

	if (pEffect = nil) then
  begin
		Result:= E_INVALIDARG;
    Exit;
  end;

  // Search through the list of registered semantics and remove all items
	// assigned to the given effect
	for iSemantic := Low(TDXUTSemantic) to High(TDXUTSemantic) do
	begin
		for iObject := Low(TDXUTObject) to High(TDXUTObject) do
		begin
			for iIndex := 0 to MAX_INDEX-1 do
			begin
				pBinding := m_Bindings[iSemantic][iObject][iIndex];

				// Clear nested arrays first
				for iParamList := 0 to Length(pBinding) - 1 do
				begin
					rParamList := @pBinding[iParamList];

					if (rParamList.pEffect = pEffect) then TParamList_Reset(rParamList^);
				end;
			end;
		end;
	end;

	Result:= S_OK;
end;


//-------------------------------------------------------------------------------------
function CDXUTEffectMap.SetWorldMatrix(const pWorldMatrix: TD3DXMatrixA16;
  const strUnits: PWideChar): HRESULT;
begin
  m_matWorld := pWorldMatrix;
  Result:= UpdateTransforms(DXUT_World);
end;


//-------------------------------------------------------------------------------------
function CDXUTEffectMap.SetViewMatrix(const pViewMatrix: TD3DXMatrixA16;
  const strUnits: PWideChar): HRESULT;
begin
  m_matView := pViewMatrix;
  Result:= UpdateTransforms(DXUT_View);
end;


//-------------------------------------------------------------------------------------
function CDXUTEffectMap.SetProjectionMatrix(const pProjectionMatrix: TD3DXMatrixA16): HRESULT;
begin
  m_matProjection := pProjectionMatrix;
  Result:= UpdateTransforms(DXUT_Projection);
end;


//-------------------------------------------------------------------------------------
function CDXUTEffectMap.UpdateTransforms(eSemantic: TDXUTSemantic;
  const strUnits: WideString): HRESULT;
var
  bEffectContains: array[Low(TDXUTSemantic)..High(TDXUTSemantic)] of Boolean;
  iTransform: TDXUTSemantic;
  matWorldInverse: TD3DXMatrixA16;
  matWorldInverseTranspose: TD3DXMatrixA16;
  matWorldView: TD3DXMatrixA16;
  matWorldViewInverse: TD3DXMatrixA16;
  matWorldViewInverseTranspose: TD3DXMatrixA16;
  matWorldViewProjection: TD3DXMatrixA16;
  matWorldViewProjectionInverse: TD3DXMatrixA16;
  matWorldViewProjectionInverseTranspose: TD3DXMatrixA16;
  matViewInverse: TD3DXMatrixA16;
  matViewInverseTranspose: TD3DXMatrixA16;
  matViewProjection: TD3DXMatrixA16;
  matViewProjectionInverse: TD3DXMatrixA16;
  matViewProjectionInverseTranspose: TD3DXMatrixA16;
  matProjectionInverse: TD3DXMatrixA16;
  matProjectionInverseTranspose: TD3DXMatrixA16;
begin
  // Determine which transforms are required by the effect
  FillChar(bEffectContains, SizeOf(bEffectContains), #0);
  for iTransform := DXUT_World to DXUT_ProjectionInverseTranspose do
  begin
    bEffectContains[iTransform] := (Length(m_Bindings[iTransform][DXUT_Geometry][0]) > 0);
  end;

  if (eSemantic = DXUT_World) then
  begin
    // World matrix permutations
    if (bEffectContains[DXUT_World] or
        bEffectContains[DXUT_WorldInverse] or
        bEffectContains[DXUT_WorldInverseTranspose]) then
    begin
      Result:= SetStandardParameter(DXUT_World, DXUT_Geometry, 0, @m_matWorld, 16, '', strUnits, 'World');
      if V_Failed(Result) then Exit;

      if (bEffectContains[DXUT_WorldInverse] or
          bEffectContains[DXUT_WorldInverseTranspose]) then
      begin
        D3DXMatrixInverse(matWorldInverse, nil, m_matWorld);
        Result := SetStandardParameter(DXUT_WorldInverse, DXUT_Geometry, 0, @matWorldInverse, 16, '', strUnits, 'World');
        if V_Failed(Result) then Exit;

        if (bEffectContains[DXUT_WorldInverseTranspose]) then
        begin
          D3DXMatrixTranspose(matWorldInverseTranspose, matWorldInverse);
          Result := SetStandardParameter(DXUT_WorldInverseTranspose, DXUT_Geometry, 0, @matWorldInverseTranspose, 16, '', strUnits, 'World');
          if V_Failed(Result) then Exit;
        end;
      end;
    end;
  end;

  if (eSemantic = DXUT_World) or
     (eSemantic = DXUT_View) then 
  begin
    // WorldView matrix permutations
    if bEffectContains[DXUT_WorldView] or
       bEffectContains[DXUT_WorldViewInverse] or
       bEffectContains[DXUT_WorldViewInverseTranspose] then
    begin
      D3DXMatrixMultiply(matWorldView, m_matWorld, m_matView);
      Result := SetStandardParameter(DXUT_WorldView, DXUT_Geometry, 0, @matWorldView, 16, '', strUnits, 'World');
      if V_Failed(Result) then Exit;

      if bEffectContains[DXUT_WorldViewInverse]  or
         bEffectContains[DXUT_WorldViewInverseTranspose] then
      begin
        D3DXMatrixInverse(matWorldViewInverse, nil, matWorldView);
        Result := SetStandardParameter(DXUT_WorldViewInverse, DXUT_Geometry, 0, @matWorldViewInverse, 16, '', strUnits, 'World');
        if V_Failed(Result) then Exit;

        if bEffectContains[DXUT_WorldViewInverseTranspose] then
        begin
          D3DXMatrixTranspose(matWorldViewInverseTranspose, matWorldViewInverse);
          Result := SetStandardParameter(DXUT_WorldViewInverseTranspose, DXUT_Geometry, 0, @matWorldViewInverseTranspose, 16, '', strUnits, 'World');
          if V_Failed(Result) then Exit;
        end;
      end;
    end;
  end;

  if eSemantic in [DXUT_World, DXUT_View, DXUT_Projection] then
  begin
    // WorldViewProjection matrix permutations
    if bEffectContains[DXUT_WorldViewProjection] or
       bEffectContains[DXUT_WorldViewProjectionInverse] or
       bEffectContains[DXUT_WorldViewProjectionInverseTranspose] then
    begin
      D3DXMatrixMultiply(matWorldViewProjection, m_matWorld, m_matView);
      D3DXMatrixMultiply(matWorldViewProjection, matWorldViewProjection, m_matProjection);

      Result := SetStandardParameter(DXUT_WorldViewProjection, DXUT_Geometry, 0, @matWorldViewProjection, 16, '', strUnits, 'World');
      if V_Failed(Result) then Exit;

      if bEffectContains[DXUT_WorldViewProjectionInverse] or
         bEffectContains[DXUT_WorldViewProjectionInverseTranspose] then
      begin
        D3DXMatrixInverse(matWorldViewProjectionInverse, nil, matWorldViewProjection);
        Result := SetStandardParameter(DXUT_WorldViewProjectionInverse, DXUT_Geometry, 0, @matWorldViewProjectionInverse, 16, '', strUnits, 'World');
        if V_Failed(Result) then Exit;

        if bEffectContains[DXUT_WorldViewProjectionInverseTranspose] then
        begin
          D3DXMatrixTranspose(matWorldViewProjectionInverseTranspose, matWorldViewProjectionInverse);
          Result := SetStandardParameter(DXUT_WorldViewProjectionInverseTranspose, DXUT_Geometry, 0, @matWorldViewProjectionInverseTranspose, 16, '', strUnits, 'World');
          if V_Failed(Result) then Exit;
        end;
      end;
    end;
  end;

  if (eSemantic = DXUT_View) then
  begin
    // View matrix permutations
    if bEffectContains[DXUT_View] or
       bEffectContains[DXUT_ViewInverse] or
       bEffectContains[DXUT_ViewInverseTranspose] then
    begin
      Result := SetStandardParameter(DXUT_View, DXUT_Geometry, 0, @m_matView, 16, '', strUnits, 'World');
      if V_Failed(Result) then Exit;

      if bEffectContains[DXUT_ViewInverse] or
         bEffectContains[DXUT_ViewInverseTranspose] then
      begin
        D3DXMatrixInverse(matViewInverse, nil, m_matView);
        Result := SetStandardParameter(DXUT_ViewInverse, DXUT_Geometry, 0, @matViewInverse, 16, '', strUnits, 'World');
        if V_Failed(Result) then Exit;

        if bEffectContains[DXUT_ViewInverseTranspose] then
        begin
          D3DXMatrixTranspose(matViewInverseTranspose, matViewInverse);
          Result := SetStandardParameter(DXUT_ViewInverseTranspose, DXUT_Geometry, 0, @matViewInverseTranspose, 16, '', strUnits, 'World');
          if V_Failed(Result) then Exit;
        end;
      end;
    end;
  end;

  if eSemantic in [DXUT_View, DXUT_Projection] then
  begin
    // ViewProjection matrix permutations
    if bEffectContains[DXUT_ViewProjection] or
       bEffectContains[DXUT_ViewProjectionInverse] or
       bEffectContains[DXUT_ViewProjectionInverseTranspose] then
    begin
      D3DXMatrixMultiply(matViewProjection, m_matView, m_matProjection);
      Result := SetStandardParameter(DXUT_ViewProjection, DXUT_Geometry, 0, @matViewProjection, 16, '', strUnits, 'World');
      if V_Failed(Result) then Exit;

      if bEffectContains[DXUT_ViewProjectionInverse] or
         bEffectContains[DXUT_ViewProjectionInverseTranspose] then
      begin
        D3DXMatrixInverse(matViewProjectionInverse, nil, matViewProjection);
        Result := SetStandardParameter(DXUT_ViewProjectionInverse, DXUT_Geometry, 0, @matViewProjectionInverse, 16, '', strUnits, 'World');
        if V_Failed(Result) then Exit;

        if bEffectContains[ DXUT_ViewProjectionInverseTranspose] then
        begin
          D3DXMatrixTranspose(matViewProjectionInverseTranspose, matViewProjectionInverse);
          Result := SetStandardParameter(DXUT_ViewProjectionInverseTranspose, DXUT_Geometry, 0, @matViewProjectionInverseTranspose, 16, '', strUnits, 'World');
          if V_Failed(Result) then Exit;
        end;
      end;
    end;
  end;

  if (eSemantic = DXUT_Projection) then
  begin
    // Projection matrix permutations
    if bEffectContains[DXUT_Projection] or
       bEffectContains[DXUT_ProjectionInverse] or
       bEffectContains[DXUT_ProjectionInverseTranspose] then
    begin
      Result := SetStandardParameter(DXUT_Projection, DXUT_Geometry, 0, @m_matProjection, 16, '', strUnits, 'World');
      if V_Failed(Result) then Exit;

      if bEffectContains[DXUT_ProjectionInverse] or
         bEffectContains[DXUT_ProjectionInverseTranspose] then
      begin
        D3DXMatrixInverse(matProjectionInverse, nil, m_matProjection);
        Result := SetStandardParameter(DXUT_ProjectionInverse, DXUT_Geometry, 0, @matProjectionInverse, 16, '', strUnits, 'World');
        if V_Failed(Result) then Exit;

        if bEffectContains[DXUT_ProjectionInverseTranspose] then
        begin
          D3DXMatrixTranspose(matProjectionInverseTranspose, matProjectionInverse);
          Result:= SetStandardParameter(DXUT_ProjectionInverseTranspose, DXUT_Geometry, 0, @matProjectionInverseTranspose, 16, '', strUnits, 'World');
          if V_Failed(Result) then Exit;
        end;
      end;
    end;
  end;

  Result:= S_OK;
end;

end.

